@@ -28,16 +28,20 @@ from expiringdict import ExpiringDict
 
 logger = logging.getLogger(__name__)
 
+mailparser_logger = logging.getLogger("mailparser")
+mailparser_logger.setLevel(logging.CRITICAL)
+
 null_file = open(os.devnull, "w")
 
 markdown_maker = html2text.HTML2Text()
 markdown_maker.unicode_snob = True
 markdown_maker.decode_errors = "replace"
 markdown_maker.body_width = 0
+markdown_maker.protect_links = True
 
 
 class EmailParserError(RuntimeError):
-    """Raised when an error parsing the email occurs"""
+    """Raised when an email parsing error occurs"""
 
 
 def decode_base64(data: str) -> bytes:
@@ -122,16 +126,24 @@ def is_outlook_msg(content: bytes) -> bool:
 
 def convert_outlook_msg(msg_bytes: bytes) -> str:
     """
-    Uses the ``msgconvert`` Perl utility to convert an Outlook MS file to
+    Uses the ``msgconvert`` Perl utility to convert an Outlook .msg file to
     standard RFC 822 format
 
+    .. warning::
+      Anomalies are introduced during conversion that make the results
+      unsuitable for forensic analysis.
+
     Args:
         msg_bytes: the content of the .msg file
 
     Returns: A RFC 822 string
     """
+    url = "https://seanthegeek.github.io/mailsuite/" \
+          "#email-samples-and-outlook-clients"
     if not is_outlook_msg(msg_bytes):
-        raise ValueError("The supplied bytes are not an Outlook MSG file")
+        raise ValueError("The supplied bytes are not an Outlook .msg file")
+    logger.warning(f"Converting Outlook .msg file for parsing. Results are not"
+                   f"suitable for forensics  See {url} for more details.")
     orig_dir = os.getcwd()
     tmp_dir = tempfile.mkdtemp()
     os.chdir(tmp_dir)
@@ -144,8 +156,8 @@ def convert_outlook_msg(msg_bytes: bytes) -> str:
         with open(eml_path, "r") as eml_file:
             rfc822 = eml_file.read()
     except FileNotFoundError:
-        raise EmailParserError(
-            "Failed to convert Outlook MSG: msgconvert utility not found")
+        raise EmailParserError("Failed to convert Outlook .msg file: "
+                               "msgconvert utility not found")
     finally:
         os.chdir(orig_dir)
         shutil.rmtree(tmp_dir)
@@ -239,9 +251,10 @@ def parse_dkim_signature(dkim_signature: Union[str, List]) -> Union[Dict,
         parts = dkim_signature_.split(";")
         for part in parts:
             key_value = part.split("=")
-            key = key_value[0].strip()
-            value = key_value[1].strip()
-            parsed_signature[key] = value
+            if len(key_value) == 2:
+                key = key_value[0].strip()
+                value = key_value[1].strip()
+                parsed_signature[key] = value
 
         if "h" in parsed_signature:
             parsed_signature["h"] = parsed_signature["h"].split(":")
@@ -281,6 +294,18 @@ def parse_email(data: Union[str, bytes],
       ``binary: True`` use ``mailsuite.utils.decode_base64`` to convert the
       payload to bytes.
     """
+    def _test_header_value(header_name: str,
+                           header_value: Union[str, int, float],
+                           startswith: bool = False) -> bool:
+        if header_name.lower() not in parsed_email:
+            return False
+        if parsed_email[header_name] is None:
+            return False
+        if startswith and all([isinstance(header_value, str),
+                               isinstance(parsed_email[header_name], str)]):
+            return parsed_email[header_name].startswith(header_value)
+        return parsed_email[header_name] == header_value
+
     if type(data) == str:
         if os.path.exists(data):
             with open(data, "rb") as f:
@@ -296,6 +321,15 @@ def parse_email(data: Union[str, bytes],
     headers_str = data.split("\n\n")[0]
     parsed_email["raw_headers"] = headers_str
     headers_str = re.sub(r"\n\s+", " ", headers_str)
+    if "subject" in parsed_email:
+        headers_str = re.sub(r"Subject: .+",
+                             f"Subject: {parsed_email['subject']}",
+                             headers_str)
+
+    if "thread-topic" in parsed_email:
+        headers_str = re.sub(r"Thread-Topic: .+",
+                             f"Thread-Topic: {parsed_email['thread-topic']}",
+                             headers_str)
     parsed_email["headers_string"] = headers_str
     from_domain = None
     if parsed_email["from"] is not None:
@@ -329,16 +363,17 @@ def parse_email(data: Union[str, bytes],
                 f"Failed to parse authentication header: {e}")
     if "body" not in parsed_email or parsed_email["body"] is None:
         parsed_email["body"] = ""
+        parsed_email["body_markdown"] = ""
     parsed_email["raw_body"] = parsed_email["body"]
     parsed_email["text_plain"] = _parsed_email.text_plain.copy()
     parsed_email["text_html"] = _parsed_email.text_html.copy()
     if len(parsed_email["text_plain"]) > 0:
         parsed_email["body"] = "\n\n".join(parsed_email["text_plain"])
-        parsed_email["body_markdown"] = "\n\n".join(parsed_email["text_plain"])
+        parsed_email["body_markdown"] = parsed_email["body"]
     if len(parsed_email["text_html"]) > 0:
         parsed_email["body"] = "\n\n".join(parsed_email["text_html"])
-        parsed_email["body_markdown"] = markdown_maker.handle(
-            parsed_email["body"])
+        parsed_email["body_markdown"] = markdown_maker.handle(parsed_email[
+                                                                  "body"])
 
     if "received" in parsed_email:
         for received in parsed_email["received"]:
@@ -425,9 +460,10 @@ def parse_email(data: Union[str, bytes],
     if "body" not in parsed_email:
         parsed_email["body"] = None
         parsed_email["body_markdown"] = None
-    else:
-        parsed_email["body_markdown"] = markdown_maker.handle(
-            parsed_email["body"])
+
+    auto_reply = all([_test_header_value("x-auto-response-suppress", "All"),
+                      _test_header_value("auto-submitted", "auto_generated")])
+    parsed_email["automatic_reply"] = auto_reply
 
     return parsed_email
 
@@ -515,8 +551,10 @@ def from_trusted_domain(message: Union[str, IOBase, Dict],
                     return True
         if "dmarc" in results:
             if "dmarc" in results:
-                dmarc = results["dkim"]
+                dmarc = results["dmarc"]
                 dmarc_result = dmarc["result"]
+                if "header.from" not in dmarc:
+                    return False
                 domain = dmarc["header.from"].lower().strip()
                 sld = publicsuffix2.get_sld(domain)
                 if dmarc_result == "pass" and domain in trusted_domains:
